@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 3461 2008-01-14 23:04:16Z richb $"
-__version__   = "$Revision: 3461 $"
-__date__      = "$Date: 2008-01-14 18:04:16 -0500 (Mon, 14 Jan 2008) $"
+__id__        = "$Id: flat_review.py 3560 2008-02-11 17:35:26Z wwalker $"
+__version__   = "$Revision: 3560 $"
+__date__      = "$Date: 2008-02-11 12:35:26 -0500 (Mon, 11 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -702,7 +702,7 @@ class Context:
                 zone = line.zones[zoneToCheck]
                 currentLineIndex = lineToCheck
                 currentZoneIndex = zoneToCheck
-                if caretOffset:
+                if caretOffset and zone.words:
                     currentWordIndex = len(zone.words) - 1
                     currentCharIndex = \
                           zone.words[currentWordIndex].length - 1
@@ -871,11 +871,12 @@ class Context:
 
         debug.println(debug.LEVEL_FINEST, "  looking at text:")
 
-        text = accessible.queryText()
-        if text:
-            zones = []
-        else:
+        try:
+            text = accessible.queryText()
+        except NotImplementedError:
             return []
+        else:
+            zones = []
 
         length = text.characterCount
 
@@ -941,6 +942,22 @@ class Context:
             textZones = self.splitTextIntoZones(
                 accessible, string, startOffset, cliprect)
 
+            # We need to account for the fact that newlines at the end of 
+            # text are treated as being on the same line when they in fact
+            # are a whole separate blank line.  So, we check for this and
+            # make up a new text zone for these cases.  See bug 434654.
+            #
+            if (endOffset == length) and (string[-1:] == "\n"):
+                [x, y, width, height] = text.getRangeExtents(startOffset, 
+                                                             endOffset,
+                                                             0)
+                if not textZones:
+                    textZones = []
+                textZones.append(TextZone(accessible,
+                                          endOffset,
+                                          "",
+                                          x, y + height, 0, height))
+
             if textZones:
                 zones.extend(textZones)
             elif len(zones):
@@ -1637,6 +1654,9 @@ class Context:
                         regionWithFocus.cursorOffset += \
                             len(zone.words[wordIndex].string.decode("UTF-8"))
                 regionWithFocus.cursorOffset += self.charIndex
+                regionWithFocus.repositionCursor()
+                if isinstance(regionWithFocus, braille.ReviewComponent):
+                    regionWithFocus.expandRegion()
                 break
 
         return [regions, regionWithFocus]
